#include "message.h"
#include "utils.h"
#include <iostream>
#include <cstdlib>

// global variables
string context; // contains the chain of called functions
list_of_msg mess; // list of error, warning messages
string st_in="dans";

// static variables
language msg::lang=francais;
string msg::type[4]= {"ERREUR","AVERTISSEMENT","",""};


// context management
void push_context(string s) {
    context+="->"+s;
}

void pop_context() {
    size_t p=context.rfind("->");

    if (p!=string::npos) {
        context.erase(p);
    }
}

// main function of messages initialisation
void init_messages(language l) {
    push_context("init_messages");
    msg::lang=l;

    switch (l) {
        case francais :
            init_french_messages();
            break;
        case english  :
            init_english_messages();
            break;
        default :
            init_english_messages();
            break;
    }

    pop_context();
}

// add a new message
void list_of_msg::add(const string & key, msg_type t, const string & m) {
    msg_data md;

    if (mess.find(key)!=mess.end()) {
        message_event("double_key_msg",md<<key);
    }

    mess[key]=msg(key,t,m);
}

// deal with the message e
//    e : msg identifier
//    d : extra data for the msg
void message_event(const string & e, const msg_data & d) {
    if (mess.find(e)==mess.end()) {
        message_event("msg_not_found",msg_data(e));
    }

    msg& m=mess[e];
    string t=msg::type[m.mt];
    
	// replace data in format
    string f=m.mes;
    uint n=d.s_data.size();
    size_t p=0,i=0;

    while (i<n && p!=string::npos) {
        p=f.find("%s",p);

        if (p==string::npos) {
            break;
        }

        f.replace(p,2,d.s_data[i]);
        i++;
    }

    n=d.i_data.size();
    p=0;
    i=0;

    while (i<n && p!=string::npos) {
        p=f.find("%i",p);

        if (p==string::npos) {
            break;
        }

        f.replace(p,2,int2str(d.i_data[i]));
        i++;
    }

    n=d.f_data.size();
    p=0;
    i=0;

    while (i<n && p!=string::npos) {
        p=f.find("%f",p);

        if (p==string::npos) {
            break;
        }

        f.replace(p,2,float2str(d.i_data[i]));
        i++;
    }

    if (m.mt==error || m.mt==warning) {
        cout << "=========================================================================" << endl;
        cout << t << " " << e << " " << st_in << " " << context << endl;
        cout << f << endl;
        cout << "=========================================================================" << endl;

    } else {
        cout << f << endl;;
    }

    if (m.mt==error) {
        exit(-1);
    }
}

// simple calls
void message_event(const string & e, const string & s) {
    message_event(e,msg_data(s));
}
void message_event(const string & e, const char * s) {
    message_event(e,msg_data(s));
}
void message_event(const string & e, double d) {
    message_event(e,msg_data(d));
}
void message_event(const string & e, int i) {
    message_event(e,msg_data(i));
}

// message definitions in french
void init_french_messages() {
    push_context("init_french_messages");
    mess.add("msg_not_found",error,"la cle de message %s n'existe pas");
    mess.add("double_key_msg",error,"la cle de message %s existe deja");
    mess.add("err_filein",error,"le fichier d'entree %s doit avoir l'extension .mel ou .msh.");
    mess.add("err_filein2",error,"il n'est pas possible d'importer un fichier %s");
    mess.add("err_filein3",error,"le format %s ne supporte que des elements d'ordre 1!");
    mess.add("err_filein4",error,"le format %s n'autorise pas des elements de types differents");
    mess.add("err_filein5",error,"aucune extension dans le fichier %s");
    mess.add("err_melelem",error,"le format mel ne supporte pas les elements %s.\nConsulter la doc de votre mailleur (gmsh ?) pour le forcer a definir les elements pleins.");
    mess.add("err_fileout",error,"le dernier argument doit etre un des formats : mel, msh, vtk, amdba ou un fichier avec une de ces extensions");
    mess.add("err_inout",error,"le fichier d'entree et la sortie sont du meme type; il n'y a rien a convertir");
    mess.add("open_fail",error,"impossible d'ouvrir le fichier %s");
    mess.add("err_dim_dom",error,"dimension anormale d'un domaine");
    mess.add("err_load_mel1",error,"le mot cle DESCRIPTION n'a pas ete trouve dans le fichier %s");
    mess.add("err_load_mel2",error,"le mot cle BLOC n'a pas ete trouve dans le fichier %s");
    mess.add("err_load_mel3",error," les maillages d'ordre > 2 ne sont pas pris en compte");
    mess.add("err_load_mel4",error,"type d'un element non identifie dans une description BLOC");
    mess.add("err_load_mel5",error,"impossible de lire le domaine %s");
    mess.add("warn_save_mel1",warning,"le domaine %s etant multidimensionnel n'est pas exporte dans le format mel.");
    mess.add("warn_save_mel2",warning,"le domaine %s n'est pas converti au format mel, les sous-domaines de dimension n-2 ne sont pas encore geres.");
    mess.add("warn_save_mel3",warning,"le domaine %s n'est pas converti au format mel, les sous-domaines point en 3D ne sont pas encore geres.");
    mess.add("err_format",error,"le fichier %s n'est pas au format %s");
    mess.add("file_confirmation",question,"le fichier %s existe deja. Voulez-vous l'ecraser (o/y ou n)?");
    mess.add("out_of_list",error,"recherche d'un point en dehors de la liste des noeuds");
    mess.add("out_of_list2",error,"recherche d'un element %i en dehors de la liste des elements");
    mess.add("err_fileext",error,"pas d'extension trouvee dans le nom de fichier %s");
    mess.add("err_fileext2",error,"extension de fichier de maillage %s non reconnue");
    mess.add("info_read",information,"Lecture du fichier %s\n");
    mess.add("info_save",information,"Ecriture du fichier %s\n");
    mess.add("err_amdba1",error,"on ne peut pas importer un fichier amdba");
    mess.add("warn_amdba",warning,"le noeud %i appartient a plusieurs domaines !");
    mess.add("err_f77_format",error,"format d'entree/sortie fortran non reconnu : %s");
    string s="le maillage contient plusieurs domaines (non pris en compte par le format vtk) : %s \n Voulez-vous sauvez ? ";
    s+="\n   0) juste le domaine principal \n   1) tous les domaines dans plusieurs fichiers (nomsauve_nomdomaine.vtk)\n Taper 0 ou 1.";
    mess.add("que_vtk_domain",question,s);
    mess.add("meshdim_null",error,"Maillage de dimension 0 !");
    mess.add("choice_i",information,"Ignorer quel domaine ? ");
    mess.add("choice_k",information,"Conserver quel domaine ? ");
    mess.add("choice_r1",information,"Renommer quel domaine ? ");
    mess.add("choice_r2",information,"Nouveau nom : ");
    mess.add("que_domains",question,"\nVous pouvez : C(ontinuer), Q(uitter), L(ister), I(gnorer), G(arder), R(enommer), E(xtraire), F(issurer) ?");
    mess.add("que_domains2",warning,"%s n'est pas reconnu comme un nom de domaine !");
    mess.add("que_domains3",warning,"%s n'est pas un domaine frontiere !");
    mess.add("que_domains4",warning,"le domaine %s existe deja, choisissez un autre nom !");
    mess.add("que_domains5",warning,"lors d'une nouvelle definition, le domaine %s doit etre un domaine d'origine !");
    mess.add("que_domains6",warning,"lors d'une definition d'une fissure, le domaine %s doit etre la frontiere de un ou deux domaines !");
    mess.add("err_constructor_1D",error,"nombre de points incorrect : nbx= 0 !");
    mess.add("err_constructor_2D",error,"nombre de points incorrect : nbx= %f nby= %f !");
    mess.add("err_crack1",error,"erreur fatale dans le process de fissuration, un element frontiere n'est pas partage par deux elements !");
    mess.add("err_crack2",error,"il n'est pas encore possible de fissurer un sous domaine impliquant deux domaines !");
    pop_context();
}

// message definitions in english
void init_english_messages() {
    push_context("init_english_messages");
    msg::type[0]="ERROR";
    msg::type[1]="WARNING";
    st_in="in";
    mess.add("msg_not_found",error,"the message key %s does not exist");
    mess.add("double_key_msg",error,"the message key %s is already defined");
    mess.add("err_filein",error,"the filename in %s has to be a .mel or .msh file");
    mess.add("err_filein2",error,"it is not possible to import %s file");
    mess.add("err_filein3",error,"the %s format does not allow elements of order > 1");
    mess.add("err_filein4",error,"elements with different types are not allowed in format %s");
    mess.add("err_filein5",error,"no extension found in file %s");
    mess.add("err_melelem",error,"the mel format does not support %s elements. Please read the user guide of your mesh generator to get the complete elements.");
    mess.add("err_fileout",error,"the last argument must be one of mel, msh, vtk, amdba or a file with one of these extensions");
    mess.add("err_inout",error,"the file in and out are of same type, nothing to convert ");
    mess.add("open_fail",error,"unable to open file %s");
    mess.add("err_dim_dom",error,"unexpected dimension of a domain");
    mess.add("err_load_mel1",error,"the keyword DESCRIPTION has not been found in the file %s");
    mess.add("err_load_mel2",error,"the keyword BLOC has not been found in the file %s");
    mess.add("err_load_mel3",error," the meshes of order > 2 are not accepted");
    mess.add("err_load_mel4",error,"element type unknown in a BLOC description");
    mess.add("err_load_mel5",error,"the domaine %s is unreadable");
    mess.add("warn_save_mel1",warning,"the domain %s having multiple definition is not exported in mel format.");
    mess.add("warn_save_mel2",warning,"domain %s is ignored in mel conversion, (n-2) dimension subdomain are not processed yet.");
    mess.add("warn_save_mel3",warning,"domain %s is ignored in mel conversion, point subdomain in 3D are not processed yet.");
    mess.add("err_format",error,"the file %s is not of the format %s");
    mess.add("file_confirmation",information,"the file %s exists. Do you want to overwrite it (y/o or n)?");
    mess.add("out_of_list",error,"search a point outside the nodes list");
    mess.add("out_of_list2",error,"the element %i is outside the element list");
    mess.add("err_fileext",error,"no extension found in the name of the file %s");
    mess.add("err_fileext2",error,"mesh file extension %s unknown");
    mess.add("info_read",information,"read file %s");
    mess.add("info_save",information,"write file %s");
    mess.add("warn_amdba",warning,"node %i belongs to two domains at least !");
    mess.add("err_f77_format",error,"input/output f77 format not identified : %s");
    string s="the mesh has more than one domain (vtk format does not manage multiple domains) : %s \n what do you want to export : ";
    s+="\n0)   only the main domain \n1)   all domains in different files (savename_domainname.vtk) \n Choose 0 or 1 ?";
    mess.add("que_vtk_domain",question,s);
    mess.add("meshdim_null",error,"the mesh dimension is 0 !");
    mess.add("choice_i",information,"Which domain do you want to ignore ? ");
    mess.add("choice_k",information,"Which domain do you want to keep ? ");
    mess.add("choice_r1",information,"Which domain do you want to rename ? ");
    mess.add("choice_r2",information,"New name : ");
    mess.add("que_domains",question,"\nYou can : C(ontinue), Q(uit), L(ist), I(gnore), K(eep), R(ename), E(xtract), D(isjoint) (meaning crack) ?");
    mess.add("que_domains2",warning,"%s is not recognized as a domain name");
    mess.add("que_domains3",warning,"%s is not a boundary domain !");
    mess.add("que_domains4",warning,"the domain %s already exists, choose an other name !");
    mess.add("que_domains5",warning,"in new definition, the domain %s has to be an original domain !");
    mess.add("que_domains6",warning,"in definition of a crack, the domain %s has to be a boundary of one or two domains !");
    mess.add("err_constructor_1D",error,"error in number of point : nbx= 0 !");
    mess.add("err_constructor_2D",error,"error in number of point : nbx= %f nby= %f !");
    mess.add("err_crack1",error,"error in cracking process, a boundary element is not shared by two elements !");
    mess.add("err_crack2",error,"it is not yet possible to crack a subdomain involving more than one domain !");
    pop_context();
}
